@@ -1,5 +1,5 @@
 package Bot::Cobalt::Plugin::YouTube;
-our $VERSION = '0.002';
+our $VERSION = '0.003000';
 
 use Bot::Cobalt;
 use Bot::Cobalt::Common;
@@ -14,7 +14,7 @@ use URI::Escape;
 
 sub REGEX () { 0 }
 
-sub new { 
+sub new {
   bless [
     qr{(youtu\.be|youtube\.com)/(\S+)},  ## ->[REGEX]
   ], shift
@@ -25,11 +25,12 @@ sub Cobalt_register {
 
   register( $self, 'SERVER', qw/
     public_msg
+    ctcp_action
     youtube_plug_resp_recv
   / );
 
   logger->info("YouTube plugin registered ($VERSION)");
-  
+
   PLUGIN_EAT_NONE
 }
 
@@ -37,7 +38,7 @@ sub Cobalt_unregister {
   my ($self, $core) = @_;
 
   logger->info("YouTube plugin unregistered.");
-  
+
   PLUGIN_EAT_NONE
 }
 
@@ -49,6 +50,33 @@ sub _create_yt_link {
       . uri_escape($id)
 }
 
+sub _issue_yt_request {
+  my ($self, $msg, $base, $id) = @_;
+
+  unless (core()->Provided->{www_request}) {
+    logger->warn(
+      "We appear to be missing Bot::Cobalt::Plugin::WWW; ",
+      "it may not be possible to issue async HTTP requests."
+    );
+  }
+
+  my $chcfg = core->get_channels_cfg( $msg->context );
+  my $this_chcfg = $chcfg->{ $msg->channel } // {};
+  return if $this_chcfg->{no_yt_retrieve};
+
+  my $req_url = $self->_create_yt_link($base, $id);
+
+  logger->debug("dispatching request to $req_url");
+
+  broadcast( 'www_request',
+    HTTP::Request->new( GET => $req_url ),
+    'youtube_plug_resp_recv',
+    [ $req_url, $msg ],
+  );
+
+  1
+}
+
 sub Bot_public_msg {
   my ($self, $core) = splice @_, 0, 2;
   my $msg = ${ $_[0] };
@@ -56,29 +84,25 @@ sub Bot_public_msg {
   my ($base, $id) = $msg->stripped =~ $self->[REGEX] ;
 
   if ($base && defined $id) {
-    unless (core()->Provided->{www_request}) {
-      logger->warn(
-        "We appear to be missing Bot::Cobalt::Plugin::WWW; ",
-        "it may not be possible to issue async HTTP requests."
-      );
-    }
-    
-    my $req_url = $self->_create_yt_link($base, $id);
-
-    logger->debug("dispatching request to $req_url");
-
-    my $request = HTTP::Request->new(
-      GET => $req_url,
-    );
-    
-    broadcast( 'www_request',
-      HTTP::Request->new( GET => $req_url ),
-      'youtube_plug_resp_recv',
-      [ $req_url, $msg ],
-    );
+    $self->_issue_yt_request($msg, $base, $id)
   }
 
-  PLUGIN_EAT_NONE  
+  PLUGIN_EAT_NONE
+}
+
+sub Bot_ctcp_action {
+  my ($self, $core) = splice @_, 0, 2;
+  my $msg = ${ $_[0] };
+
+  return PLUGIN_EAT_NONE unless $msg->channel;
+
+  my ($base, $id) = $msg->stripped =~ $self->[REGEX];
+
+  if ($base && defined $id) {
+    $self->_issue_yt_request($msg, $base, $id) 
+  }
+
+  PLUGIN_EAT_NONE
 }
 
 sub Bot_youtube_plug_resp_recv {
@@ -98,20 +122,24 @@ sub Bot_youtube_plug_resp_recv {
 
   my ($title, $short_url);
 
-  while (my $tok = $html->get_tag('meta', 'link') ) {
-    my $args = ref $tok->[1] eq 'HASH' ? $tok->[1] : next ;
-    
+  TAG: while (my $tok = $html->get_tag('meta', 'link') ) {
+    my $args = ref $tok->[1] eq 'HASH' ? $tok->[1] : next TAG ;
+
     if (defined $args->{name} && $args->{name} eq 'title') {
       $title = $args->{content}
     }
-    
+
     if (defined $args->{rel} && $args->{rel} eq 'shortlink') {
       $short_url = $args->{href}
     }
+
+    if (defined $title && defined $short_url) {
+      last TAG
+    }
   }
 
   if (defined $title && $short_url) {
-    my $irc_resp = 
+    my $irc_resp =
       color('bold', 'YouTube:')
       . " $title ( $short_url )" ;
 
@@ -143,11 +171,13 @@ Bot::Cobalt::Plugin::YouTube - YouTube plugin for Bot::Cobalt
 
 A L<Bot::Cobalt> plugin.
 
-Retrieves YouTube links pasted to an IRC channel and reports titles 
+Retrieves YouTube links pasted to an IRC channel and reports titles
 (as well as shorter urls) to IRC.
 
 Operates on both 'youtube.com' and 'youtu.be' links.
 
+Disregards channels with a 'no_yt_retrieve' flag enabled.
+
 =head1 AUTHOR
 
 Jon Portnoy <avenj@cobaltirc.org>
